#
# ZProtocol2000 : Zope Kramer's Protocol2000 Support
#
# (c) 2003 Jerome Alet - <alet@unice.fr>
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#

import urllib

from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import Implicit
from Globals import DTMLFile, Persistent, InitializeClass, MessageDialog
from AccessControl.Role import RoleManager
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

import jaxml
from pavgrid import Protocol2000Error, Protocol2000, ACTION_AUDIO_FOLLOW_VIDEO, ACTION_AUDIO_BREAKAWAY

manage_addZProtocol2000Form = DTMLFile('dtml/ZProtocol2000Add', globals())

version = "1.0"

_Grid = None        # Our Protocol2000 machines

def manage_addZProtocol2000(self, id, title='', serialport='/dev/tts/0', timeout=1.0, REQUEST=None) :
    """Adds the ZProtocol2000 object in the current Folder."""
    ref = ZProtocol2000(id, title, serialport, timeout)
    dest = self.Destination()
    dest._setObject(id, ref)
    if REQUEST is not None:
        try:    url=self.DestinationURL()
        except: url=REQUEST['URL1']
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)
    return id

class ZProtocol2000Error(Exception):
    """An exception for Zope Protocol 2000 support."""
    def __init__(self, message = ""):
        self.message = message
        Exception.__init__(self, message)
    def __repr__(self):
        return self.message
    __str__ = __repr__
    
class ZProtocol2000(Implicit, Persistent, SimpleItem, PropertyManager) :
    """ZProtocol2000 implementation."""

    _properties = (
                      {'id':'title', 'type': 'string', 'mode':'w'},
                      {'id':'serialport', 'type': 'string', 'mode':'w'},
                      {'id':'timeout', 'type': 'float', 'mode':'w'},
                  )

    meta_type = "ZProtocol2000"
    
    security = ClassSecurityInfo()
    security.declareObjectProtected('View')
    
    def __init__(self, id, title = '', serialport='/dev/tts/0', timeout=1.0) :
        """Initialization."""
        self.id = id
        self.title = title.strip()
        self.serialport = serialport.strip()    
        self.timeout = float(timeout)

    manage_options = PropertyManager.manage_options + ( 
                       { "label" : "View", "action" : "index_html" },
                     ) + SimpleItem.manage_options

    security.declareProtected('View', 'index_html')
    def index_html(self, REQUEST=None) :
        """Displays the Switcher's status."""
        global _Grid
        self.init_grid()
        xml = jaxml.HTML_document()
        xml._push()
        xml.form(action="%s/gridActions" % self.this().absolute_url(), method="POST")
        for machine in range(1, len(_Grid.Machines) + 1) :
            m = _Grid.Machines[machine]
            broken = self._isBrokenAway(m)
            m.define_machine_number_of_video_inputs()
            try :
                ninputs = m.numberOfVideoInputs
            except AttributeError :    
                ninputs = 1
            m.define_machine_number_of_video_outputs()
            try :
                noutputs = m.numberOfVideoOutputs
            except AttributeError :    
                noutputs = 1
            av = "v"
            for part in range(0, broken + 1) :
                if not broken :
                    prefix = "Audio/Video"
                else :    
                    if not part :
                        prefix = "Video"
                    else :    
                        prefix = "Audio"
                        av = "a"
                        m.define_machine_number_of_audio_inputs()
                        ninputs = m.numberOfAudioInputs
                        m.define_machine_number_of_audio_outputs()
                        noutputs = m.numberOfVideoInputs
                        
                xml.h3("%s Machine %i" % (prefix, machine))
                xml._push()
                xml.table(border=1, cellpadding=4)
                
                xml._push()
                xml.tr()
                xml._push()
                xml.th()
                xml._push()
                xml._submit(name="action_reset%s" % av, value="Reset %s" % prefix)
                xml._pop()
                xml._br()
                xml._submit(name="action_alloff%s" % av, value="Switch all %s Off" % prefix)
                xml._pop()
                xml.th("Outputs", colspan=noutputs + 1)
                xml._pop()
                
                xml._push()
                xml.tr().th("Inputs", rowspan=ninputs + 1)
                xml._push()
                xml.th()
                if m.hasBreakawaySetting :
                    if not broken :
                        xml._submit(name="action_breakav%i" % machine, value="Split Audio/Video")
                    else :    
                        xml._submit(name="action_joinav%i" % machine, value="Join Audio/Video")
                xml._pop()
                for output in range(1, noutputs + 1) :
                    xml._push()
                    xml.th()
                    xml._text("%i" % output)
                    xml._br()
                    xml._submit(value="Off", name="action_off%sm%i/i0/o%i" % (av, machine, output))
                    xml._pop()
                    if not part :
                        m.request_video_output_status(output=output)
                    else :
                        m.request_audio_output_status(output=output)
                xml._pop()
                
                for input in range(1, ninputs + 1) :
                    xml._push()
                    xml.tr()
                    xml.th("%i" % input)
                    for output in range(1, noutputs + 1) :
                        try :
                            if not part :
                                status = m.VideoOutputs[output]
                            else :    
                                status = m.AudioOutputs[output]
                            if status == input :     
                                color = "green"
                            else :    
                                color = "red"
                            xml._push()
                            xml.td(bgcolor=color, align="center", valign="middle")
                            xml._submit(value="O", name="action_switch%sm%i/i%i/o%i" % (av, machine, input, output))
                            xml._pop()
                        except AttributeError :    
                            pass
                    xml._pop()
                xml._pop()
        xml._pop()
        return '\n'.join(str(xml).split('\n')[1:])
        
    security.declareProtected('View', '__call__')
    __call__ = index_html    
        
    security.declareProtected('View', 'init_grid')
    def init_grid(self, force=0) :            
        """Initializes the Protocol2000 machines if needed."""
        global _Grid
        if (_Grid is None) or force :
            if _Grid is not None :
                _Grid.close() # if serial port is in open state, close it first.
            _Grid = Protocol2000(device=self.serialport, timeout=self.timeout)
            
    def manage_beforeDelete(self, item, container):
        """Closes the link on deletion from the ZODB."""
        global _Grid
        if _Grid is not None :
            _Grid.close()
            _Grid = None
                
    security.declareProtected('Control Protocol2000 Grid', 'doAction')
    def doAction(self, name) :
        """Executes an action on the grid.
         
           The action name contains its parameters, example :
           
             switchvm1/i2/o4
             
           means :  
           
             Switch Video Input 2 to Video Output 4 on Machine 1.
        """
        valid_actions = [method[7:] for method in dir(self.__class__) if method.startswith("action_") and not method.endswith("__roles__")]
        name = name.strip()
        params = None
        found = 0
        for action in valid_actions :
            lg = len(action)
            if name.startswith(action) :
                params = name[lg:]
                found = 1
                break
        if found :        
            try :
                method = getattr(self, "action_%s" % action)
            except AttributeError :
                raise ZProtocol2000Error, "Unknown error (%s, %s, %s), please report bug" % (name, action, valid_actions)
            else :    
                method(params)
        else :        
            raise ZProtocol2000Error, "Unknown action %s, valid actions are %s" % (name, str(valid_actions))
            
    security.declareProtected('Control Protocol2000 Grid', 'gridActions')
    def gridActions(self, REQUEST=None) :
        """Launches actions on the Grid."""
        if REQUEST is not None :
            actionstodo = [action[7:] for action in REQUEST.form.keys() if action.startswith("action_")]
            changed = 0
            for action in actionstodo :
                self.doAction(action)
                changed = 1
            if changed :    
                REQUEST.RESPONSE.redirect(REQUEST["HTTP_REFERER"]) # go back to where we were before
                
                
    security.declareProtected('Control Protocol2000 Grid', 'action_alloffv')
    def action_alloffv(self, params) :    
        """Switches all Video Off."""
        # WARNING : This doesn't really work, but switching
        # off each output individually works. Seems to be a bug
        # in our VS-5X4
        global _Grid
        self.init_grid()
        for machine in range(1, len(_Grid.Machines) + 1) :
            _Grid.Machines[machine].disconnect_video()
            
    security.declareProtected('Control Protocol2000 Grid', 'action_alloffa')
    def action_alloffa(self, params) :    
        """Switches all Audio Off."""
        # WARNING : This doesn't really work, but switching
        # off each output individually works. Seems to be a bug
        # in our VS-5X4
        global _Grid
        self.init_grid()
        for machine in range(1, len(_Grid.Machines) + 1) :
            _Grid.Machines[machine].disconnect_audio()
        
    security.declareProtected('Control Protocol2000 Grid', 'action_offv')
    def action_offv(self, params) :    
        """Switches Video Off."""
        global _Grid
        self.init_grid()
        destination = self._splitSwitcherParams(params)
        if destination is not None :
            (machine, input, output) = destination
            try :
                _Grid.Machines[machine].disconnect_video(output)
            except KeyError :    
                raise ZProtocol2000Error, "Invalid machine number %i" % machine
            
    security.declareProtected('Control Protocol2000 Grid', 'action_offa')
    def action_offa(self, params) :    
        """Switches Audio Off."""
        global _Grid
        self.init_grid()
        destination = self._splitSwitcherParams(params)
        if destination is not None :
            (machine, input, output) = destination
            try :
                _Grid.Machines[machine].disconnect_audio(output)
            except KeyError :    
                raise ZProtocol2000Error, "Invalid machine number %i" % machine
        
    security.declareProtected('Control Protocol2000 Grid', 'action_breakav')
    def action_breakav(self, params) :    
        """Breaks Audio/Video apart."""
        global _Grid
        self.init_grid()
        machine = _Grid.Machines[int(params)]
        machine.set_audio_breakaway()
        
    security.declareProtected('Control Protocol2000 Grid', 'action_joinav')
    def action_joinav(self, params) :    
        """Joins Audio/Video on a given machine."""
        global _Grid
        self.init_grid()
        machine = _Grid.Machines[int(params)]
        machine.set_audio_follow_video()
        
    security.declareProtected('Control Protocol2000 Grid', 'action_reseta')
    def action_reseta(self, params) :
        """Resets audio."""
        global _Grid
        self.init_grid()
        for machine in range(1, len(_Grid.Machines) + 1) :
            _Grid.Machines[machine].reset_audio()
    
    security.declareProtected('Control Protocol2000 Grid', 'action_resetv')
    def action_resetv(self, params) :
        """Resets video."""
        global _Grid
        self.init_grid()
        for machine in range(1, len(_Grid.Machines) + 1) :
            _Grid.Machines[machine].reset_video()
    
    security.declareProtected('Control Protocol2000 Grid', 'action_switcha')
    def action_switcha(self, params) :
        """Switches an audio input to an audio output."""
        global _Grid
        self.init_grid()
        destination = self._splitSwitcherParams(params)
        if destination is not None :
            (machine, input, output) = destination
            try :
                _Grid.Machines[machine].switch_audio(input, output)
            except KeyError :    
                raise ZProtocol2000Error, "Invalid machine number %i" % machine
    
    security.declareProtected('Control Protocol2000 Grid', 'action_switchv')
    def action_switchv(self, params) :
        """Switches a video input to a video output."""
        global _Grid
        self.init_grid()
        destination = self._splitSwitcherParams(params)
        if destination is not None :
            (machine, input, output) = destination
            try :
                _Grid.Machines[machine].switch_video(input, output)
            except KeyError :    
                raise ZProtocol2000Error, "Invalid machine number %i" % machine
                
    def _isBrokenAway(self, machine) :        
        """Returns the break away status for a given machine."""
        machine.has_breakaway_setting()
        try :
            if machine.hasBreakawaySetting :
                machine.request_breakaway_setting()
                if machine.currentBreakawaySetting == ACTION_AUDIO_FOLLOW_VIDEO :
                    return 0
                elif machine.currentBreakawaySetting == ACTION_AUDIO_BREAKAWAY :    
                    return 1
                else :    
                    # unknown
                    return 0
            else :    
                return 0
        except AttributeError :        
            machine.hasBreakawaySetting = 0
            return 0
            
    def _splitSwitcherParams(self, params) :        
        """Splits the switcher parameters and returns a tuple (machine, input, output)."""
        parts = params.split('/')
        if len(parts) != 3 :
            raise ZProtocol2000Error, "Incomplete switch parameter %s" % params
        else :    
            try :
                return [int(part[1:]) for part in parts]
            except ValueError :    
                raise ZProtocol2000Error, "Invalid switch parameter %s" % params
    
InitializeClass(ZProtocol2000)
